@@ -1,11 +1,11 @@
 from flask.ext.restful import Resource, fields
-from flask import request
+from flask import request, abort, Response
 import inspect
 import functools
 import re
 from flask_restful_swagger import registry, registered, api_spec_endpoint
-from flask_restful_swagger import html
-
+from jinja2 import Template
+import os
 
 resource_listing_endpoint = None
 
@@ -31,15 +31,18 @@ def docs(api, apiVersion='0.0', swaggerVersion='1.2',
     register_once(api_add_resource, apiVersion, swaggerVersion, basePath,
                   resourcePath, produces, api_spec_url)
     return api_add_resource(resource, path, *args, **kvargs)
+
   api.add_resource = add_resource
 
   return api
 
+rootPath = os.path.dirname(__file__)
 
 def register_once(add_resource_func, apiVersion, swaggerVersion, basePath,
                   resourcePath, produces, endpoint):
   global registered
   global api_spec_endpoint
+  global api_spec_static
   global resource_listing_endpoint
   if not registered:
     registered = True
@@ -55,7 +58,64 @@ def register_once(add_resource_func, apiVersion, swaggerVersion, basePath,
     add_resource_func(SwaggerRegistry, ep, endpoint=ep)
     resource_listing_endpoint = endpoint + '/_/resource_list.json'
     add_resource_func(ResourceLister, resource_listing_endpoint, endpoint=resource_listing_endpoint)
+    api_spec_static = endpoint + '/_/static/'
+    add_resource_func(StaticFiles, api_spec_static + '<string:dir1>/<string:dir2>/<string:dir3>', api_spec_static + '<string:dir1>/<string:dir2>', api_spec_static + '<string:dir1>')
 
+templates = {}
+
+def render_endpoint(endpoint):
+  return render_page("endpoint.html", endpoint.__dict__)
+
+def render_homepage(resource_list_url):
+  conf = {'resource_list_url': resource_list_url}
+  return render_page("index.html", conf)
+
+def render_page(page, info):
+  url = registry['basePath']
+  if url.endswith('/'):
+    url = url.rtrim('/')
+  conf = {'base_url': api_spec_static, 'full_base_url': url + api_spec_static}
+  if info != None:
+    conf.update(info)
+  global templates
+  if templates.has_key(page):
+    template = templates[page]
+  else:
+    fs = open(os.path.join(rootPath, 'static', page), "r")
+    template = Template(fs.read())
+    templates[page] = template
+  mime = 'text/html'
+  if page.endswith('.js'):
+    mime = 'text/javascript'
+  return Response(template.render(conf), mimetype=mime)
+
+class StaticFiles(Resource):
+  def get(self, dir1 = None, dir2 = None, dir3 = None):
+    if dir1 == None:
+      filePath = "index.html"
+    else:
+      filePath = dir1
+      if dir2 != None:
+        filePath = "%s/%s" % (filePath, dir2)
+        if dir3 != None:
+          filePath = "%s/%s" % (filePath, dir3)
+    if filePath in ["index.html", "o2c.html", "swagger-ui.js", "swagger-ui.min", "lib/swagger-oauth.js"]:
+      conf = {'resource_list_url': api_spec_endpoint}
+      return render_page(filePath, conf)
+    mime = 'text/plain'
+    if filePath.endswith(".gif"):
+      mime = 'image/gif'
+    elif filePath.endswith(".png"):
+      mime = 'image/png'
+    elif filePath.endswith(".js"):
+      mime = 'text/javascript'
+    elif filePath.endswith(".css"):
+      mime = 'text/css'
+    filePath = os.path.join(rootPath, 'static', filePath)
+    if os.path.exists(filePath):
+      fs = open(filePath, "r")
+      return Response(fs, mimetype=mime)
+    abort(404)
 
 class ResourceLister(Resource):
   def get(self):
@@ -80,7 +140,7 @@ def swagger_endpoint(resource, path):
       if request.path.endswith('.help.json'):
         return endpoint.__dict__
       if request.path.endswith('.help.html'):
-        return html.render_endpoint(endpoint)
+        return render_endpoint(endpoint)
   return SwaggerResource
 
 
@@ -94,8 +154,13 @@ class SwaggerEndpoint(object):
   @staticmethod
   def extract_operations(resource, path_arguments=[]):
     operations = []
-    for method in resource.methods:
-      method_impl = resource.__dict__[method.lower()]
+    for method in [m.lower() for m in resource.methods]:
+      method_impl = resource.__dict__.get(method, None)
+      if method_impl is None:
+        for cls in resource.__mro__:
+          for item_key in cls.__dict__.keys():
+            if item_key == method:
+              method_impl = cls.__dict__[item_key]
       op = {
           'method': method,
           'parameters': path_arguments,
@@ -131,7 +196,7 @@ def merge_parameter_list(base, override):
 class SwaggerRegistry(Resource):
   def get(self):
     if request.path.endswith('.html'):
-      return html.render_homepage(resource_listing_endpoint)
+      return render_homepage(resource_listing_endpoint)
     return registry
 
 
@@ -189,6 +254,10 @@ def add_model(model_class):
     # resource_fields.
     # If that attribute exists then we deduce the swagger model by the content
     # of this attribute
+
+    if hasattr(model_class, 'required'):
+      required = model['required'] = model_class.required
+
     properties = model['properties'] = {}
     nested = model_class.nested() if isinstance(model_class, _Nested) else {}
     for field_name, field_type in list(model_class.resource_fields.items()):
